network = "testnet"
host = "localhost"


class ServiceConfig:
    def __init__(self, address, datadir):
        self.address = address
        self.datadir = datadir


bitcoind = ServiceConfig("http://xu:xu@$host:18332", "$datadir/bitcoind")
litecoind = ServiceConfig("http://xu:xu@$host:19332", "$datadir/litecoind")
lndbtc = ServiceConfig("grpc://$host:10009", "$datadir/lndbtc")
lndltc = ServiceConfig("grpc://$host:20009", "$datadir/lndltc")
geth = ServiceConfig("http://$host:8545", "$datadir/geth")
raiden = ServiceConfig("http://$host:5001", "$datadir/raiden")
xud = ServiceConfig("http://$host:8886", "$datadir/xud")






import grpc
from . import xurpc_pb2 as xu
from .xurpc_pb2_grpc import XudStub
import logging


class Xud:
    def __init__(self, connstr, cert):
        creds = grpc.ssl_channel_credentials(cert)
        connstr = connstr.replace('grpc://', '')
        channel = grpc.secure_channel(connstr, creds)
        self.client = XudStub(channel)

    @property
    def status(self):
        try:
            self.client.GetInfo(xu.GetInfoRequest())
            return "OK"
        except grpc.RpcError as e:
            return "Error: (gRPC) %s" % e.details()
        except Exception as e:
            return "Error: %s" % type(e)



import platform
import os
from .rpc.bitcoind import Bitcoind
from .rpc.lnd import Lnd
from .rpc.raiden import Raiden
from .rpc.xud import Xud
from .rpc.geth import Geth
from . import config
import re

# Hide CryptographyDeprecationWarning: encode_point has been deprecated on EllipticCurvePublicNumbers and will be removed in a future version. Please use EllipticCurvePublicKey.public_bytes to obtain both compressed and uncompressed point encoding.
#   m.add_string(self.Q_C.public_numbers().encode_point())
# Ref. https://stackoverflow.com/a/55336269/2663029
import warnings
warnings.filterwarnings(action='ignore',module='.*paramiko.*')

prefix = "XUD_DOCKER_"

HOME = prefix + "HOME"

networks = {
    "regtest": {
        "services": {}
    },
    "simnet": {
        "services": {}
    },
    "testnet": {
        "services": {}
    },
    "mainnet": {
        "services": {}
    }
}


def print_services(title, services):
    c1w = max(len(title[0]), len(max(services.keys(), key=len)))
    c2w = max(len(title[1]), len(max(services.values(), key=len)))
    c1f = '{:' + str(c1w) + "}"
    c2f = '{:' + str(c2w) + "}"

    print(("┏━━" + c1f + "━━┯━━" + c2f + "━━┓").format("━" * c1w, "━" * c2w))
    print(("┃  " + c1f + "  │  " + c2f + "  ┃").format(title[0], title[1]))

    for key, value in services.items():
        print(("┠──" + c1f + "──┼──" + c2f + "──┨").format("─" * c1w, "─" * c2w))
        print(("┃  " + c1f + "  │  " + c2f + "  ┃").format(key, value))

    print(("┗━━" + c1f + "━━┷━━" + c2f + "━━┛").format("━" * c1w, "━" * c2w))


def get_home():
    """
    :return:
    Linux
        /home/<user>/.xud-docker
    Darwin (macOS)
        /Users/<user>/Library/Application Support/XudDocker
    Windows
        C:\\Users\\<user>\\AppData\\Local\\XudDocker
    """
    if HOME in os.environ:
        return os.environ[HOME]

    if platform.system() == 'Linux':
        return os.path.expanduser("~/.xud-docker")
    elif platform.system() == 'Darwin':
        return os.path.expanduser("~/Library/Application Support/XudDocker")
    elif platform.system() == 'Windows':
        return os.path.expanduser("~/AppData/Local/XudDocker")


def load(path):
    try:
        if ":" in path:
            # client = paramiko.SSHClient()
            # client.load_system_host_keys()
            #
            # m = re.search(r'^(.+)@(.+):(.+)$', path)
            # if m is None:
            #     raise Exception("Failed to load content from " + path)
            # path = m.group(3)
            # user = m.group(1)
            # host = m.group(2)
            #
            # client.connect(host, timeout=1, username=user)
            # sftp = client.open_sftp()
            # return sftp.open(path).read()
            raise Exception("Remote loading is not supported")
        else:
            return open(path, 'rb').read()
    except FileNotFoundError:
        raise Exception("Failed to load content from " + path)


def expand(path):
    if config.host == 'localhost':
        return path
    else:
        return config.host + ":" + path


def build_service(name):
    network = config.network

    if name == 'bitcoind':
        service = Bitcoind(config.bitcoind.address)
    elif name == 'litecoind':
        service = Bitcoind(config.litecoind.address)
    elif name == 'lndbtc':
        macaroon = expand(config.lndbtc.datadir) + ('/data/chain/bitcoin/%s/admin.macaroon' % network)
        cert = expand(config.lndbtc.datadir) + '/tls.cert'
        service = Lnd(config.lndbtc.address, load(cert), load(macaroon))
    elif name == 'lndltc':
        macaroon = expand(config.lndltc.datadir) + ('/data/chain/litecoin/%s/admin.macaroon' % network)
        cert = expand(config.lndltc.datadir) + '/tls.cert'
        service = Lnd(config.lndltc.address, load(cert), load(macaroon))
    elif name == 'geth':
        service = Geth(config.geth.address)
    elif name == 'raiden':
        service = Raiden(config.raiden.address)
    elif name == 'xud':
        cert = expand(config.xud.datadir) + '/tls.cert'
        service = Xud(config.xud.address, load(cert))
    else:
        raise Exception('Unsupported service name: ' + name)

    return service


def get_service(name):
    network = config.network
    if network not in networks:
        raise Exception("Unsupported network: %s" % network)
    services = networks[network]["services"]
    if name in services:
        return services[name]

    s = build_service(name)
    services[name] = s

    return s





import requests
from requests.exceptions import ConnectionError
import logging

class Raiden:
    def __init__(self, connstr):
        self.prefix = connstr
        pass

    @property
    def status(self):
        try:
            r = requests.get(self.prefix + "/api/v1/address")
            logging.info('Raiden status: %s', r)
            return "OK"
        except ConnectionError:
            return "(connecting...)"
        except Exception as e:
            return "Error: %s" % type(e)



import argparse
import os
from .utils import get_home
import pathlib
import logging
import subprocess
from . import config

logging.basicConfig(level=logging.INFO)

from .up import main as up
from .status import main as status
from .down import main as down
from .shell import main as shell


# os.chdir("/Users/yy/work/huiyin/xud-docker-dev/xud-testnet")
# os.system("docker-compose up -d")
#
# time.sleep(3)
#
# print(w3.eth.blockNumber)
# print(w3.eth.syncing)
# os.system("docker-compose down")
# os.system("docker-compose up -d geth")
# os.system("docker-compose exec -T geth geth --testnet --exec 'admin.nodeInfo' attach")
# os.system("docker-compose exec -T geth geth --testnet --exec 'eth.accounts' attach")
# os.system("docker-compose exec -T geth geth --testnet --exec 'eth.syncing' attach")
# os.system("docker-compose exec -T geth geth --testnet --exec 'eth.blockNumber' attach")
# os.system("docker-compose exec -T geth geth --testnet account list")
# os.system("docker-compose logs --tail=200 geth")
# output = subprocess.check_output("pwd")
# print(output)
# output = subprocess.check_output("docker-compose exec geth geth --testnet --help", shell=True)
# print(output)
# docker-compose exec geth geth --testnet attach
# personal.createAccount()


def check_docker():
    p = subprocess.Popen('which docker', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    p.wait()
    stdout, stderr = p.communicate()
    r = p.returncode
    if r != 0:
        logging.error("docker is missing")
        exit(1)
    # os.system('docker --version')


def check_docker_compose():
    p = subprocess.Popen('which docker', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    p.wait()
    stdout, stderr = p.communicate()
    r = p.returncode
    if r != 0:
        logging.error("docker-compose is missing")
        exit(1)
    # os.system('docker-compose --version')


def init_parser_with_config(parser):
    parser.add_argument("-n", "--network", type=str, default=config.network,
                        choices=['regtest', 'simnet', 'testnet', 'mainnet'])
    parser.add_argument("--bitcoind.address", type=str, default=config.bitcoind.address)
    parser.add_argument("--bitcoind.datadir", type=str, default=config.bitcoind.datadir)
    parser.add_argument("--litecoind.address", type=str, default=config.litecoind.address)
    parser.add_argument("--litecoind.datadir", type=str, default=config.litecoind.datadir)
    parser.add_argument("--lndbtc.address", type=str, default=config.lndbtc.address)
    parser.add_argument("--lndbtc.datadir", type=str, default=config.lndbtc.datadir)
    parser.add_argument("--lndltc.address", type=str, default=config.lndltc.address)
    parser.add_argument("--lndltc.datadir", type=str, default=config.lndltc.datadir)
    parser.add_argument("--geth.address", type=str, default=config.geth.address)
    parser.add_argument("--geth.datadir", type=str, default=config.geth.datadir)
    parser.add_argument("--raiden.address", type=str, default=config.raiden.address)
    parser.add_argument("--raiden.datadir", type=str, default=config.raiden.datadir)
    parser.add_argument("--xud.address", type=str, default=config.xud.address)
    parser.add_argument("--xud.datadir", type=str, default=config.xud.datadir)

    parser.add_argument("--host", type=str, default=config.host)


def update_config(args):
    config.network = args["network"]
    config.bitcoind.address = args["bitcoind.address"]
    config.bitcoind.datadir = args["bitcoind.datadir"]
    config.litecoind.address = args["litecoind.address"]
    config.litecoind.datadir = args["litecoind.datadir"]
    config.lndbtc.address = args["lndbtc.address"]
    config.lndbtc.datadir = args["lndbtc.datadir"]
    config.lndltc.address = args["lndltc.address"]
    config.lndltc.datadir = args["lndltc.datadir"]
    config.geth.address = args["geth.address"]
    config.geth.datadir = args["geth.datadir"]
    config.raiden.address = args["raiden.address"]
    config.raiden.datadir = args["raiden.datadir"]
    config.xud.address = args["xud.address"]
    config.xud.datadir = args["xud.datadir"]

    config.host = args["host"]


def main():
    parser = argparse.ArgumentParser()

    init_parser_with_config(parser)

    sub = parser.add_subparsers(dest="subparser")
    cmd_up = sub.add_parser("up")
    cmd_down = sub.add_parser("down")
    cmd_status = sub.add_parser("status")
    cmd_shell = sub.add_parser("shell")

    args = parser.parse_args()

    kwargs = vars(args)

    update_config(kwargs)

    # Make sure docker is installed
    #check_docker()

    # Make sure docker-compose is installed
    #check_docker_compose()

    # home = get_home() + "/" + args.network
    #
    # if not os.path.exists(home):
    #     pathlib.Path(home).mkdir(parents=True, exist_ok=True)
    #
    # if not os.path.isdir(home):
    #     logging.error("{} is not a directory", home)
    #     exit(1)
    #
    # # Go to xud-docker home directory
    # os.chdir(home)

    try:
        # globals()[kwargs.pop('subparser')](**kwargs)
        globals()[kwargs.pop('subparser')]()
    except KeyError:
        parser.print_help()


if __name__ == '__main__':
    main()
